#include <Alert.h>
#include <Bitmap.h>
#include <Deskbar.h>
#include <Dragger.h>
#include <Messenger.h>
#include <Roster.h>
#include <Window.h>
#include <stdio.h>


#include "VolumeControl.h"
#include "VolumeSlider.h"
#include "iconfile.h"


const char *app_signature = "application/x-vnd.iNfO-VolumeControl";

VolumeView::VolumeView(BRect frame,uint32 resizeMask, uint32 flags)
	:	BView(frame, "VolumeControl", resizeMask, flags)
{
	// Background Bitmap
	segments = new BBitmap(BRect(0, 0, 16 - 1, 16 - 1), B_COLOR_8_BIT);
	segments->SetBits(Speaker16x16_raw, 16*16, 0, B_COLOR_8_BIT);
	// Background Color
	SetViewColor(184,184,184);

	//add dragger
	BRect rect(Bounds());
	rect.left = rect.right-7.0; 
	rect.top = rect.bottom-7.0;
	BDragger *dragger = new BDragger(rect, this, B_FOLLOW_RIGHT | B_FOLLOW_BOTTOM);
	AddChild(dragger);
	dragger->SetViewColor(B_TRANSPARENT_32_BIT);
}

VolumeView::VolumeView(BMessage *message)
	:	BView(message)
{
	// Background Bitmap
	segments = new BBitmap(BRect(0, 0, 16 - 1, 16 - 1), B_COLOR_8_BIT);
	segments->SetBits(Speaker16x16_raw, 16*16, 0, B_COLOR_8_BIT);
	// Background Color
	SetViewColor(184,184,184);

}

VolumeView::~VolumeView()
{
	delete segments;
}

// archiving overrides
VolumeView *
VolumeView::Instantiate(BMessage *data)
{
	if (!validate_instantiation(data, "VolumeView"))
		return NULL;
	return new VolumeView(data);
}

status_t 
VolumeView::Archive(BMessage *data, bool deep) const
{
	BView::Archive(data, deep);
	data->AddString("add_on", app_signature);
	return B_NO_ERROR;
}

void VolumeView::AttachedToWindow() 
{ 
	if ( Parent() ) 
		SetViewColor(Parent()->ViewColor()); 
	BView::AttachedToWindow(); 
		
}

void 
VolumeView::Draw(BRect rect)
{
//	BView::Draw(rect);
	SetDrawingMode(B_OP_OVER);	//Select?
	DrawBitmap(segments);
}


void 
VolumeView::MouseDown(BPoint point)
{
	if (modifiers() & (B_CONTROL_KEY | B_COMMAND_KEY))
	{
		(new BAlert("Coming Soon!!", "Coming Soon: VolumeControl Setup\n\n"
					"Soon you will be able to reach VolumeControl's "
					"setup by holding down CONTROL or ALT while "
					"clicking. If you have a three buttoned mouse "
					"you can use the third button instead.\n"
					"\n"
					"If you were trying to launch the Media preferences " 
					"try right-clicking instead.\n","Shut up!"))->Go();
		return;
	}
	
	uint32 mouseButtons;
	BPoint where;
	GetMouse(&where, &mouseButtons, true);	
	switch (mouseButtons)
	{
		case B_PRIMARY_MOUSE_BUTTON:
		{
			// Show VolumeSlider
			ConvertToScreen(&point);
			(new VolumeSlider(BRect(point.x,point.y,point.x+20,point.y+20)))->Show();
		}
		break;
		case B_SECONDARY_MOUSE_BUTTON:
		{
			//Launch Media
			status_t	stat;
			char		*argv[ 2 ];
			argv[ 0 ] = "";
			stat = be_roster->Launch( "application/x-vnd.Be.MediaPrefs", 1, argv  );
		}
		break;
		case B_TERTIARY_MOUSE_BUTTON:
		{
			(new BAlert("Coming Soon!!", "Coming Soon: VolumeControl Setup\n\n"
						"Soon you will be able to reach VolumeControl's "
						"setup by holding down CONTROL or ALT while "
						"clicking. If you have a three buttoned mouse "
						"you can use the third button instead.\n"
						"\n"
						"If you were trying to launch the Media preferences " 
						"try right-clicking instead.\n","Shut up!"))->Go();

		}
	}
}


/*
	Shows the 'about'-box, works.
*/
void
VolumeView::MessageReceived(BMessage *message)
{
	switch(message->what){
	case B_ABOUT_REQUESTED:
		(new BAlert("About VolumeControl", "     VolumeControl v0.99 (Replicant)\n"
			"\n"
			"Usage:\n"
			"1: Doubleclick on the VolumeControl icon.\n"
			"2: (In terminal)\n"
			"VolumeControl [-deskbar]\n"
			" -deskbar		Installs VolumeControl in deskbar.\n"
			"  See 'README' for further info on this.\n\n"
			"  Brought to you by Fredrik Holmqvist.\n"
			"  E-mail: ferhol-6@sm.luth.se\n\n"
			" Made with a Be mousepad...\n\n"
			" I'm sorry for being inactive for so long!\n"
			"Sometimes work takes the best of us."
			,"Shut up!"))->Go();
			break;
	default:;

	}
}


VolumeApplication::VolumeApplication(bool install)
	:	BApplication(app_signature)
{
	if (install) {
		// install option specified, install self into the Deskbar
		BRect buttonRect(0, 0, 16, 16);
		BDeskbar db;
		db.AddItem(new VolumeView(buttonRect));
		// commit suicide
		PostMessage(B_QUIT_REQUESTED);
		return;
	}
	BRect windowRect(130, 100, 146, 116);
	BWindow *window = new BWindow(windowRect, "Volume", B_MODAL_WINDOW_LOOK, B_NORMAL_WINDOW_FEEL, 
	B_NOT_CLOSABLE | B_NOT_ZOOMABLE | B_NOT_RESIZABLE | B_WILL_ACCEPT_FIRST_CLICK );
	BRect buttonRect(window->Bounds());
	VolumeView *view = new VolumeView(buttonRect, B_FOLLOW_ALL_SIDES, B_WILL_DRAW);
	window->AddChild(view);
	window->Show();
}


/*
   main: Optimized and tested looks good.
*/
int
main(int, char **argv)
{

	int n=0;
	bool install=false;
	while (argv[++n]) {
		if (!strcmp(argv[n],"-deskbar")) install=true;
		else {
			printf(	"# %s v0.99 (c) 2002 FredriK HolmQvisT.\n"
				"# Usage:\n"
				"#  %s [-deskbar]\n"
				"#\n"
				"# Options:\n"
				"#  -deskbar		Installs VolumeControl in deskbar\n"
				"#  Comments and bugs to: ferhol-6@sm.luth.se\n"
				,argv[0], argv[0]);
			return 0;
		}
	}
	(new VolumeApplication(install))->Run();
	return 0;
}