#include <ParameterWeb.h>
#include <MediaRoster.h>
#include <MediaTheme.h>
#include <MultiChannelControl.h>
#include <Screen.h>
#include <stdio.h>

#include "VolumeSlider.h"

 uint32 vCHGMessage = 'vCHG';

VolumeSlider::VolumeSlider(BRect frame)
	: BWindow(frame, "VolumeSlider", B_BORDERED_WINDOW, B_ASYNCHRONOUS_CONTROLS | B_WILL_ACCEPT_FIRST_CLICK, 0)
{
	media_node aOutNode;

	BMediaRoster* roster = BMediaRoster::Roster();
	roster->GetAudioOutput(&aOutNode);
	roster->GetParameterWebFor(aOutNode, &paramWeb);


	//Finding the Mixer slider in the audio output ParameterWeb 
	int32 numParams = paramWeb->CountParameters();
	BParameter* p = NULL;
	for (int i = 0; i < numParams; i++)
	{
		p = paramWeb->ParameterAt(i);
		if (!strcmp(p->Name(), "Mixer")) {
			for (; i < numParams; i++) {
				p = paramWeb->ParameterAt(i);
				if (strcmp(p->Kind(), B_GAIN)) p=NULL;
				else break;
			}
			break;
		} else p = NULL;
	}
	if (p==NULL) {
		printf("Could not find the mixer.\n");
		exit(1);
	} else if(p->Type()!=BParameter::B_CONTINUOUS_PARAMETER) {
		printf("Mixer is unknown.\n");
		exit(2);
	}

	mixerParam= dynamic_cast<BContinuousParameter*>(p);
	min = mixerParam->MinValue();
	max = mixerParam->MaxValue();
	step = mixerParam->ValueStep();


	// Create the controller in the preferred theme.
	BMediaTheme* theme = BMediaTheme::PreferredTheme();
	BMultiChannelControl* mixerControl = static_cast<BMultiChannelControl*> (theme->MakeControlFor(mixerParam));

	// Add to window and resize to fit...
	AddChild(mixerControl);
	ResizeTo(mixerControl->Bounds().Width(), mixerControl->Bounds().Height());
	
	mixerControl->SetTarget(this);
	vCHGMessage = mixerControl->Command();

	//Make sure it's not outside the screen.
	const int32 kMargin = 3;
	BRect windowRect=ConvertToScreen(Bounds());
	BRect screenFrame(BScreen(B_MAIN_SCREEN_ID).Frame());
	if (screenFrame.right < windowRect.right + kMargin)
		MoveBy(- kMargin - windowRect.right + screenFrame.right, 0);
	if (screenFrame.bottom < windowRect.bottom + kMargin)
		MoveBy(0, - kMargin - windowRect.bottom + screenFrame.bottom);
	if (screenFrame.left > windowRect.left - kMargin)
		MoveBy(kMargin + screenFrame.left - windowRect.left, 0);
	if (screenFrame.top > windowRect.top - kMargin)
		MoveBy(0, kMargin + screenFrame.top - windowRect.top);

}

VolumeSlider::~VolumeSlider()
{
	delete paramWeb;
}

void
VolumeSlider::WindowActivated(bool active)
{
	if (!active)
	{
		Lock();
		Quit();
	}
}

void 
VolumeSlider::MessageReceived(BMessage *msg)
{
	msg->PrintToStream();
	if (msg->what == vCHGMessage)
	{
		float chanData[2];
		bigtime_t lastChange;
		size_t size = sizeof(chanData);
		
		mixerParam->GetValue( &chanData, &size, &lastChange );

		for( int i=0; i<2; i++) {
			bool changed = false;
			int32 sliderVal;
		
			if( msg->FindBool("be:channel_changed", i, &changed) != B_OK) return;
			if( changed ) {
				if( msg->FindInt32("be:channel_value", i, &sliderVal) !=B_OK) return;
				
				chanData[i] = min + sliderVal  * step;
				chanData[i] = chanData[i] > max ? max: chanData[i];		
			}
		}

		printf("Min value: %f      Max Value: %f\nData: %f     %f\n", mixerParam->MinValue(),mixerParam->MaxValue(), chanData[0], chanData[1]);
		mixerParam->SetValue(&chanData, sizeof(chanData), 0);
	}
	else BWindow::MessageReceived(msg);		// not a slider message, not our problem 
}

